@@ -15,6 +15,7 @@ def main(arg=None):
     def prepare_dataset(batch):
         audio = batch["audio"]
         batch["input_values"] = model.processor(audio["array"], sampling_rate=16_000).input_values[0]
+        batch["length"] = len(batch["input_values"])
         sent = batch["text"] if 'text' in batch else batch["sentence"]
         batch["labels"] = model.tokenizer(sent).input_ids[1:]
         batch["input_ids"] = batch["labels"]
@@ -42,13 +43,13 @@ def main(arg=None):
         max_length_labels: Optional[int] = None
         pad_to_multiple_of: Optional[int] = None
         pad_to_multiple_of_labels: Optional[int] = None
+        selftype: bool = False
 
         def __call__(self, features: List[Dict[str, Union[List[int], torch.Tensor]]]) -> Dict[str, torch.Tensor]:
             # split inputs and labels since they have to be of different lenghts and need
             # different padding methods
             input_features = [{"input_values": feature["input_values"]} for feature in features]
-            target_name = 'labels' if "labels" in features[0] else "input_ids"
-            label_features = [{"input_ids": feature[target_name]} for feature in features]
+            label_features = [{"input_ids": feature['labels']} for feature in features]
 
             batch = self.processor.pad(
                 input_features,
@@ -64,10 +65,13 @@ def main(arg=None):
                 max_length=self.max_length_labels,
                 pad_to_multiple_of=self.pad_to_multiple_of_labels,
                 return_tensors="pt",
-            ).input_ids
+            )
 
-            batch[target_name] = labels_batch
-            batch.pop('attention_mask', None)
+            if self.selftype:
+                labels_batch = labels_batch['input_ids']
+            else:
+                labels_batch = labels_batch['input_ids'].masked_fill(labels_batch.attention_mask.ne(1), -100)
+            batch['labels'] = labels_batch
             return batch
 
     def parse_args(args):
@@ -79,7 +83,10 @@ def main(arg=None):
         parser.add_argument("--SpeechMixSelf", action='store_true')
         parser.add_argument("--dataset", type=str)
         parser.add_argument("--field", type=str)
-        parser.add_argument("--split", type=str)
+        parser.add_argument("--train_split", type=str)
+        parser.add_argument("--test_split", type=str)
+        parser.add_argument("--grad_accum", default=3, type=str)
+        parser.add_argument("--batch", type=int)
         parser.add_argument("--ftl", action='store_true')
         parser.add_argument("--lna", action='store_true')
         parser.add_argument("--lnae", action='store_true')
@@ -113,8 +120,8 @@ def main(arg=None):
         model = speechmix.SpeechMixED(input_arg['speech_model_config'], input_arg['nlp_model_config'],
                                       ftl=input_arg['ftl'])
 
-    train_ds = load_dataset(input_arg["dataset"], input_arg["field"], split=input_arg["split"])
-    valid_ds = load_dataset(input_arg["dataset"], input_arg["field"], split=input_arg["split"])
+    train_ds = load_dataset(input_arg["dataset"], input_arg["field"], split=input_arg["train_split"])
+    valid_ds = load_dataset(input_arg["dataset"], input_arg["field"], split=input_arg["test_split"])
 
     train_ds = train_ds.cast_column("audio", Audio(sampling_rate=16_000))
     valid_ds = valid_ds.cast_column("audio", Audio(sampling_rate=16_000))
@@ -122,26 +129,32 @@ def main(arg=None):
     train_ds = train_ds.map(prepare_dataset, num_proc=1)
     valid_ds = valid_ds.map(prepare_dataset, num_proc=1)
 
-    data_collator = DataCollatorWithPadding(processor=model.processor, tokenizer=model.tokenizer, padding=True)
+    print("selftype", (model_type == 'SpeechMixSelf'))
+    data_collator = DataCollatorWithPadding(processor=model.processor, tokenizer=model.tokenizer, padding=True,
+                                            selftype=(model_type == 'SpeechMixSelf'))
 
     training_args = TrainingArguments(
         output_dir=f"./{input_arg['speech_model_config']}_{input_arg['nlp_model_config']}_{model_type}",
-        per_device_train_batch_size=6,
-        per_device_eval_batch_size=6,
-        gradient_accumulation_steps=5,
+        per_device_train_batch_size=int(input_arg['batch']),
+        per_device_eval_batch_size=int(input_arg['batch']),
+        gradient_accumulation_steps=int(input_arg['grad_accum']),
         eval_accumulation_steps=2,
         evaluation_strategy="steps",
         group_by_length=True,
-        num_train_epochs=60,
-        fp16=False,
+        load_best_model_at_end=True,
+        num_train_epochs=1000,
+        fp16=True,
         save_steps=500,
         eval_steps=500,
         logging_steps=100,
         learning_rate=3e-4,
         warmup_steps=500,
         save_total_limit=2,
+        dataloader_num_workers=10,
     )
-    # some trainer problem - save all logistics on compute_metrics, cause out of memory, fix:argmax first; dynamic padding on past key value, cause index error, fix: return only loss and logist
+    # some trainer problem - save all logistics on compute_metrics, cause out of memory, fix:argmax first;
+    # dynamic padding on past key value, cause index error, fix: return only loss and logist
+    # group_by_length took lots of time during preprocessing
     trainer = Trainer(
         model=model,
         args=training_args,
